import gql from "graphql-tag";
import * as Errors from "../errors";
import * as Factories from "../factories";
import * as Models from "../models";
import { BaseQueryRequest, query } from "./Base";

// region fetchFAQs
export type FetchFAQsRequest = BaseQueryRequest;
export type FetchFAQsData = {
	faqs: Array<{
		id: number;
		title?: string;
		content?: string;
		isHTML?: boolean;
	}>;
};
export type FetchFAQsVariables = {};
export type SuccessfulFetchFAQsResponse = {
	success: true;
	faqs: Models.FAQ[];
};
export type FailedFetchFAQsResponse = {
	success: false;
	error: Errors.GenericError;
};
export type FetchFAQsResponse = SuccessfulFetchFAQsResponse | FailedFetchFAQsResponse;
export const fetchFAQs = async (request: FetchFAQsRequest): Promise<FetchFAQsResponse> => {
	const response = await query<FetchFAQsData, FetchFAQsVariables, FetchFAQsRequest>(request, {
		query: gql`
			{
				faqs {
					id
					title
					content
					isHTML
				}
			}
		`,
		variables: {}
	});

	if (response.success) {
		const { data } = response.rawResponse;

		if (data && Array.isArray(data.faqs))
			return {
				success: true,
				faqs: data.faqs.map(Factories.faqFromJSON)
			};
	}

	return {
		success: false,
		error: Errors.genericError("Hubo un error al intentar obtener la lista de faqs.")
	};
};
// endregion
