import { action } from "mobx";
import * as moment from "moment";
import { assign, omitNullable, pick } from "../helpers";
import * as Models from "../models";
import { courseClassListFromJSON, CourseClassListJSON } from "./CourseClassList";
import { userFromJSON, UserJSON } from "./User";

export type CourseJSON = {
	id: number;
	code: string;
	name?: Nullable<string>;
	iconURL?: Nullable<string>;
	eva?: Nullable<string>;
	semester?: Nullable<number>;
	year?: Nullable<number>;

	classLists?: Nullable<CourseClassListJSON[]>;

	createdAt?: Nullable<string>;
	createdBy?: Nullable<UserJSON>;
	updatedAt?: Nullable<string>;
	updatedBy?: Nullable<UserJSON>;
	deletedAt?: Nullable<string>;
	deletedBy?: Nullable<UserJSON>;
};

export const courseFromJSON = action(
	(json: CourseJSON): Models.Course => {
		const result = new Models.Course();

		assign(result, omitNullable(pick(json, ["id", "code", "name", "iconURL", "eva", "year"])));

		if (json.semester === 1 || json.semester === 2) result.semester = json.semester;

		if (Array.isArray(json.classLists)) result.classLists = json.classLists.map(courseClassListFromJSON);

		if (json.createdAt) result.createdAt = moment(json.createdAt);
		if (json.createdBy) result.createdBy = userFromJSON(json.createdBy);

		if (json.updatedAt) result.updatedAt = moment(json.updatedAt);
		if (json.updatedBy) result.updatedBy = userFromJSON(json.updatedBy);

		if (json.deletedAt) result.deletedAt = moment(json.deletedAt);
		if (json.deletedBy) result.deletedBy = userFromJSON(json.deletedBy);

		return result;
	}
);
