import { action } from "mobx";
import * as moment from "moment";
import { assign, omitNullable, pick } from "../helpers";
import * as Models from "../models";
import { courseClassListFromJSON, CourseClassListJSON } from "./CourseClassList";
import { userFromJSON, UserJSON } from "./User";
import { videoFromJSON, VideoJSON } from "./Video";

export type CourseClassJSON = {
	id: number;
	number?: Nullable<number>;
	title?: Nullable<string>;
	disabled?: Nullable<boolean>;

	courseClassList?: Nullable<CourseClassListJSON>;
	videos?: Nullable<Array<VideoJSON>>;

	createdAt?: Nullable<string>;
	createdBy?: Nullable<UserJSON>;
	updatedAt?: Nullable<string>;
	updatedBy?: Nullable<UserJSON>;
	deletedAt?: Nullable<string>;
	deletedBy?: Nullable<UserJSON>;
};

export const courseClassFromJSON = action(
	(json: CourseClassJSON): Models.CourseClass => {
		const result = new Models.CourseClass();

		assign(result, omitNullable(pick(json, ["id", "number", "title", "disabled"])));

		if (Array.isArray(json.videos)) result.videos = json.videos.map(videoFromJSON);
		if (json.courseClassList) result.courseClassList = courseClassListFromJSON(json.courseClassList);

		if (json.createdAt) result.createdAt = moment(json.createdAt);
		if (json.createdBy) result.createdBy = userFromJSON(json.createdBy);

		if (json.updatedAt) result.updatedAt = moment(json.updatedAt);
		if (json.updatedBy) result.updatedBy = userFromJSON(json.updatedBy);

		if (json.deletedAt) result.deletedAt = moment(json.deletedAt);
		if (json.deletedBy) result.deletedBy = userFromJSON(json.deletedBy);

		return result;
	}
);
