import { action } from "mobx";
import * as moment from "moment";
import { assign, omitNullable, pick } from "../helpers";
import * as Models from "../models";
import { courseFromJSON, CourseJSON } from "./Course";
import { courseClassFromJSON, CourseClassJSON } from "./CourseClass";
import { userFromJSON, UserJSON } from "./User";

export type CourseClassListJSON = {
	id: number;
	name?: Nullable<string>;

	course?: Nullable<CourseJSON>;
	classes?: Nullable<Array<CourseClassJSON>>;

	createdAt?: Nullable<string>;
	createdBy?: Nullable<UserJSON>;
	updatedAt?: Nullable<string>;
	updatedBy?: Nullable<UserJSON>;
	deletedAt?: Nullable<string>;
	deletedBy?: Nullable<UserJSON>;
};

export const courseClassListFromJSON = action(
	(json: CourseClassListJSON): Models.CourseClassList => {
		const result = new Models.CourseClassList();

		assign(result, omitNullable(pick(json, ["id", "name"])));

		if (json.course) result.course = courseFromJSON(json.course);
		if (Array.isArray(json.classes)) result.classes = json.classes.map(courseClassFromJSON);

		if (json.createdAt) result.createdAt = moment(json.createdAt);
		if (json.createdBy) result.createdBy = userFromJSON(json.createdBy);

		if (json.updatedAt) result.updatedAt = moment(json.updatedAt);
		if (json.updatedBy) result.updatedBy = userFromJSON(json.updatedBy);

		if (json.deletedAt) result.deletedAt = moment(json.deletedAt);
		if (json.deletedBy) result.deletedBy = userFromJSON(json.deletedBy);

		return result;
	}
);
