import { action } from "mobx";
import * as moment from "moment";
import { assign, omitNullable, pick } from "../helpers";
import * as Models from "../models";
import { userFromJSON, UserJSON } from "./User";

export type FAQJSON = {
	id: number;
	title?: Nullable<string>;
	content?: Nullable<string>;
	isHTML?: Nullable<boolean>;

	createdAt?: Nullable<string>;
	createdBy?: Nullable<UserJSON>;
	updatedAt?: Nullable<string>;
	updatedBy?: Nullable<UserJSON>;
	deletedAt?: Nullable<string>;
	deletedBy?: Nullable<UserJSON>;
};

export const faqFromJSON = action(
	(json: FAQJSON): Models.FAQ => {
		const result = new Models.FAQ();

		assign(result, omitNullable(pick(json, ["id", "title", "content", "isHTML"])));

		if (json.createdAt) result.createdAt = moment(json.createdAt);
		if (json.createdBy) result.createdBy = userFromJSON(json.createdBy);

		if (json.updatedAt) result.updatedAt = moment(json.updatedAt);
		if (json.updatedBy) result.updatedBy = userFromJSON(json.updatedBy);

		if (json.deletedAt) result.deletedAt = moment(json.deletedAt);
		if (json.deletedBy) result.deletedBy = userFromJSON(json.deletedBy);

		return result;
	}
);
