import { action } from "mobx";
import * as moment from "moment";
import { assign, omitNullable, pick } from "../helpers";
import * as Models from "../models";
import { userRoleFromJSON, UserRoleJSON } from "./UserRole";

export type UserJSON = {
	id: number;
	email?: Nullable<string>;
	name?: Nullable<string>;

	roles?: Nullable<Array<UserRoleJSON>>;

	createdAt?: Nullable<string>;
	updatedAt?: Nullable<string>;
	deletedAt?: Nullable<string>;
};

export const userFromJSON = action(
	(json: UserJSON): Models.User => {
		const result = new Models.User();

		assign(result, omitNullable(pick(json, ["id", "email", "name"])));

		if (Array.isArray(json.roles)) result.roles = json.roles.map(userRoleFromJSON);

		if (json.createdAt) result.createdAt = moment(json.createdAt);
		if (json.updatedAt) result.updatedAt = moment(json.updatedAt);
		if (json.deletedAt) result.deletedAt = moment(json.deletedAt);

		return result;
	}
);
