import { action } from "mobx";
import * as moment from "moment";
import { assign, omitNullable, pick } from "../helpers";
import * as Models from "../models";

export type UserRoleJSON = {
	id: number;
	name?: Nullable<string>;

	createdAt?: Nullable<string>;
	updatedAt?: Nullable<string>;
	deletedAt?: Nullable<string>;
};

export const userRoleFromJSON = action(
	(json: UserRoleJSON): Models.UserRole => {
		const result = new Models.UserRole();

		assign(result, omitNullable(pick(json, ["id", "name"])));

		if (json.createdAt) result.createdAt = moment(json.createdAt);
		if (json.updatedAt) result.updatedAt = moment(json.updatedAt);
		if (json.deletedAt) result.deletedAt = moment(json.deletedAt);

		return result;
	}
);
