import { action } from "mobx";
import * as moment from "moment";
import { assign, omitNullable, pick } from "../helpers";
import * as Models from "../models";
import { userFromJSON, UserJSON } from "./User";
import { videoQualityFromJSON, VideoQualityJSON } from "./VideoQuality";

export type VideoJSON = {
	id: number;
	name?: Nullable<string>;
	position?: Nullable<number>;

	qualities?: Nullable<Array<VideoQualityJSON>>;

	createdAt?: Nullable<string>;
	createdBy?: Nullable<UserJSON>;
	updatedAt?: Nullable<string>;
	updatedBy?: Nullable<UserJSON>;
	deletedAt?: Nullable<string>;
	deletedBy?: Nullable<UserJSON>;
};

export const videoFromJSON = action(
	(json: VideoJSON): Models.Video => {
		const result = new Models.Video();

		assign(result, omitNullable(pick(json, ["id", "name", "position"])));

		if (Array.isArray(json.qualities)) result.qualities = json.qualities.map(videoQualityFromJSON);

		if (json.createdAt) result.createdAt = moment(json.createdAt);
		if (json.createdBy) result.createdBy = userFromJSON(json.createdBy);

		if (json.updatedAt) result.updatedAt = moment(json.updatedAt);
		if (json.updatedBy) result.updatedBy = userFromJSON(json.updatedBy);

		if (json.deletedAt) result.deletedAt = moment(json.deletedAt);
		if (json.deletedBy) result.deletedBy = userFromJSON(json.deletedBy);

		return result;
	}
);
