import { action } from "mobx";
import * as moment from "moment";
import { assign, omitNullable, pick } from "../helpers";
import * as Models from "../models";
import { userFromJSON, UserJSON } from "./User";

export type VideoFormatJSON = {
	id: number;
	name?: Nullable<string>;
	url?: Nullable<string>;

	createdAt?: Nullable<moment.Moment>;
	createdBy?: Nullable<UserJSON>;
	updatedAt?: Nullable<moment.Moment>;
	updatedBy?: Nullable<UserJSON>;
	deletedAt?: Nullable<moment.Moment>;
	deletedBy?: Nullable<UserJSON>;
};

export const videoFormatFromJSON = action(
	(json: VideoFormatJSON): Models.VideoFormat => {
		const result = new Models.VideoFormat();

		assign(result, omitNullable(pick(json, ["id", "name", "url"])));

		if (json.createdAt) result.createdAt = moment(json.createdAt);
		if (json.createdBy) result.createdBy = userFromJSON(json.createdBy);

		if (json.updatedAt) result.updatedAt = moment(json.updatedAt);
		if (json.updatedBy) result.updatedBy = userFromJSON(json.updatedBy);

		if (json.deletedAt) result.deletedAt = moment(json.deletedAt);
		if (json.deletedBy) result.deletedBy = userFromJSON(json.deletedBy);

		return result;
	}
);
