import { action } from "mobx";
import * as moment from "moment";
import { assign, omitNullable, pick } from "../helpers";
import * as Models from "../models";
import { userFromJSON, UserJSON } from "./User";
import { videoFormatFromJSON, VideoFormatJSON } from "./VideoFormat";

export type VideoQualityJSON = {
	id: number;
	height?: Nullable<number>;
	width?: Nullable<number>;
	disabled?: Nullable<boolean>;

	formats?: Nullable<Array<VideoFormatJSON>>;

	createdAt?: Nullable<string>;
	createdBy?: Nullable<UserJSON>;
	updatedAt?: Nullable<string>;
	updatedBy?: Nullable<UserJSON>;
	deletedAt?: Nullable<string>;
	deletedBy?: Nullable<UserJSON>;
};

export const videoQualityFromJSON = action(
	(json: VideoQualityJSON): Models.VideoQuality => {
		const result = new Models.VideoQuality();

		assign(result, omitNullable(pick(json, ["id", "height", "width", "disabled"])));

		if (Array.isArray(json.formats)) result.formats = json.formats.map(videoFormatFromJSON);

		if (json.createdAt) result.createdAt = moment(json.createdAt);
		if (json.createdBy) result.createdBy = userFromJSON(json.createdBy);

		if (json.updatedAt) result.updatedAt = moment(json.updatedAt);
		if (json.updatedBy) result.updatedBy = userFromJSON(json.updatedBy);

		if (json.deletedAt) result.deletedAt = moment(json.deletedAt);
		if (json.deletedBy) result.deletedBy = userFromJSON(json.deletedBy);

		return result;
	}
);
