export const assign = <TObject>(object: TObject, source: Partial<TObject>) => {
	Object.assign(object, source);
};

export const omitNullable = <TObject>(
	object: TObject
): { [K in keyof TObject]: TObject[K] extends null | undefined ? never : NonNullable<TObject[K]> } => {
	return Object.keys(object).reduce(
		(previousValue, currentValue) => {
			const value = object[currentValue];

			if (value !== undefined && value !== null) previousValue[currentValue] = value;

			return previousValue;
		},
		{} as { [K in keyof TObject]: TObject[K] extends null | undefined ? never : NonNullable<TObject[K]> }
	);
};

export const pick = <T, K extends keyof T>(source: T, keys: K[]): Pick<T, K> => {
	return keys.reduce<Pick<T, K>>(
		(previousValue, currentValue) => {
			previousValue[currentValue] = source[currentValue];

			return previousValue;
		},
		{} as Pick<T, K>
	);
};

export const secondsToString: (seconds: number, length?: "seconds" | "minutes" | "hours") => string = (
	seconds,
	length
) => {
	const secondsLeft = Math.floor(seconds % 60);
	const minutes = Math.floor((seconds / 60) % 60);
	const hours = Math.floor(seconds / 60 / 60);

	const fillWithZero = (n: number) => `${n < 10 ? "0" : ""}${n}`;

	let result = fillWithZero(secondsLeft);

	if (length !== "seconds") result = `${fillWithZero(minutes)}:${result}`;

	if (length !== "minutes") result = `${fillWithZero(hours)}:${result}`;

	return result;
};
