import { observable, runInAction } from "mobx";
import { useEffect, useRef, useState } from "react";
import * as Models from "../models";

export const useCourseSearch = (courses: Models.Course[]): UseCourseSearchOutput => {
	const [value, setValue] = useState("");

	const courseSearchStateRef = useRef<CourseSearchState>(
		observable<CourseSearchState>({
			value,
			results: courses
		})
	);

	useEffect(() => {
		const prepareToCompare = (value: string): string =>
			value
				.toLowerCase()
				.replace("á", "a")
				.replace("é", "e")
				.replace("í", "i")
				.replace("ó", "o")
				.replace("ú", "u");

		const results =
			value.length === 0
				? courses
				: courses.filter(course => {
						let courseNameToCompare = prepareToCompare(course.name!);

						return value.split("").every(char => {
							const index = courseNameToCompare.indexOf(prepareToCompare(char));

							courseNameToCompare =
								index > -1 && index < courseNameToCompare.length
									? courseNameToCompare.substr(index + 1)
									: "";

							return index !== -1;
						});
				  });

		runInAction(() => {
			courseSearchStateRef.current.value = value;
			courseSearchStateRef.current.results = results;
		});
	}, [value, courses]);

	return [courseSearchStateRef.current, setValue];
};

export type CourseSearchState = {
	value: string;
	results: Models.Course[];
};

export type UseCourseSearchOutput = [
	{
		value: string;
		results: Models.Course[];
	},
	(value: string) => void
];
