import { observable } from "mobx";
import { useCallback, useContext, useEffect } from "react";
import { RootStoreContext } from "src/core/context";
import {
	FetchCourseByCodeResponse,
	FetchCourseByCodeState,
	InitialFetchCourseByCodeState
} from "../stores/CourseStore";

export const useFetchCourseByCode = ({
	autoFetch,
	forceFetch,
	courseCode
}: UseFetchCourseByCodeInput): UseFetchCourseByCodeOutput => {
	const { courseStore } = useContext(RootStoreContext);

	const fetch = useCallback<() => Promise<FetchCourseByCodeResponse>>(
		() => courseStore.fetchCourseByCode({ courseCode }),
		[courseStore]
	);

	const state =
		courseStore.fetchCourseByCodeStates.get(courseCode) ||
		observable<InitialFetchCourseByCodeState>({ isLoading: false });

	useEffect(() => {
		if (autoFetch && (!state.success || forceFetch) && !state.isLoading) fetch();
	}, [courseCode]);

	return [state, fetch];
};

export type UseFetchCourseByCodeInput = {
	autoFetch: boolean;
	forceFetch: boolean;
	courseCode: string;
};
export type UseFetchCourseByCodeOutput = [FetchCourseByCodeState, () => Promise<FetchCourseByCodeResponse>];
