import { action, observable } from "mobx";
import * as Models from "../models";
import { Base } from "./Base";

export class CourseClassListStore extends Base {
	@observable public courseClassLists = new Map<number, Models.CourseClassList>();

	@action public saveCourseClassList(courseClassList: Models.CourseClassList): Models.CourseClassList {
		const { courseClassLists } = this;
		const { courseStore, courseClassStore, userStore } = this.rootStore;

		let savedCourseClassList = courseClassLists.get(courseClassList.id);

		if (savedCourseClassList === courseClassList) return courseClassList;

		if (savedCourseClassList === undefined) {
			courseClassLists.set(courseClassList.id, courseClassList);
			savedCourseClassList = courseClassLists.get(courseClassList.id)!;
		}

		// Reconciliation
		if (courseClassList.createdBy) courseClassList.createdBy = userStore.saveUser(courseClassList.createdBy);
		if (courseClassList.updatedBy) courseClassList.updatedBy = userStore.saveUser(courseClassList.updatedBy);
		if (courseClassList.deletedBy) courseClassList.deletedBy = userStore.saveUser(courseClassList.deletedBy);

		if (courseClassList.course) courseClassList.course = courseStore.saveCourse(courseClassList.course);

		if (courseClassList.classes) {
			courseClassList.classes = courseClassList.classes.map(cc => {
				const courseClass = courseClassStore.saveCourseClass(cc);
				courseClass.courseClassList = courseClassList;

				return courseClass;
			});
		}

		for (const key of Object.keys(courseClassList)) {
			const value = courseClassList[key];

			if (value !== undefined) savedCourseClassList[key] = value;
		}

		return savedCourseClassList;
	}

	@action public deleteCourseClassList(courseClassList: Models.CourseClassList | number): boolean {
		const { courseClassLists } = this;

		return courseClassLists.delete(typeof courseClassList === "number" ? courseClassList : courseClassList.id);
	}
}
