import { action, observable } from "mobx";
import * as Models from "../models";
import { Base } from "./Base";

export class CourseClassStore extends Base {
	@observable public courseClasses = new Map<number, Models.CourseClass>();

	@observable public latestCourseClasses: Models.CourseClass[] = [];

	@action public saveCourseClass(courseClass: Models.CourseClass): Models.CourseClass {
		const { courseClasses } = this;
		const { courseClassListStore, userStore } = this.rootStore;

		let result = courseClasses.get(courseClass.id);

		if (result === courseClass) return courseClass;

		if (result === undefined) {
			courseClasses.set(courseClass.id, courseClass);
			result = courseClasses.get(courseClass.id)!;
		}

		// Reconciliation
		if (courseClass.courseClassList)
			courseClass.courseClassList = courseClassListStore.saveCourseClassList(courseClass.courseClassList);
		if (courseClass.createdBy) courseClass.createdBy = userStore.saveUser(courseClass.createdBy);
		if (courseClass.updatedBy) courseClass.updatedBy = userStore.saveUser(courseClass.updatedBy);
		if (courseClass.deletedBy) courseClass.deletedBy = userStore.saveUser(courseClass.deletedBy);

		// if (courseClass.videos) courseClass.videos = courseClass.videos.map(video => videoStore.saveVideo(video));

		for (const key of Object.keys(courseClass)) {
			const value = courseClass[key];

			if (value !== undefined) result![key] = value;
		}

		return result;
	}

	@action
	public deleteCourseClass(courseClass: Models.CourseClass | number): boolean {
		const { courseClasses } = this;

		const savedCourseClass = courseClasses.get(typeof courseClass === "number" ? courseClass : courseClass.id);

		if (!savedCourseClass) return false;

		const { courseClassList } = savedCourseClass;
		if (courseClassList && courseClassList.classes) {
			const courseClassIndex = courseClassList.classes.findIndex(cc => cc.id === savedCourseClass.id);

			if (courseClassIndex !== -1) courseClassList.classes.splice(courseClassIndex, 1);
		}

		return courseClasses.delete(typeof courseClass === "number" ? courseClass : courseClass.id);
	}
}
