import { action, observable } from "mobx";
import * as Models from "../models";
import { Base } from "./Base";

export class FAQStore extends Base {
	@observable public faqs = new Map<number, Models.FAQ>();

	@observable public faqList: Models.FAQ[] = [];

	@action public saveFAQ(faq: Models.FAQ): Models.FAQ {
		const { faqs } = this;
		const { userStore } = this.rootStore;

		let savedFAQ = faqs.get(faq.id);

		if (savedFAQ === faq) return faq;

		if (savedFAQ === undefined) {
			faqs.set(faq.id, faq);
			savedFAQ = faqs.get(faq.id)!;
		}

		// Reconciliation
		if (faq.createdBy) faq.createdBy = userStore.saveUser(faq.createdBy);
		if (faq.updatedBy) faq.updatedBy = userStore.saveUser(faq.updatedBy);
		if (faq.deletedBy) faq.deletedBy = userStore.saveUser(faq.deletedBy);

		for (const key of Object.keys(faq)) {
			const value = faq[key];

			if (value !== undefined) savedFAQ[key] = value;
		}

		return savedFAQ;
	}

	@action public deleteFAQ(faq: Models.FAQ | number): boolean {
		const { faqs } = this;
		return faqs.delete(typeof faq === "number" ? faq : faq.id);
	}
}
