import { action, observable } from "mobx";
import * as Models from "../models";
import { Base } from "./Base";

export class UserStore extends Base {
	@observable public users = new Map<number, Models.User>();

	@observable public authToken: string | undefined;

	@observable public currentUser: Models.User | undefined;

	@action public setAuthToken(authToken: string | undefined) {
		this.authToken = authToken;
	}

	@action public setCurrentUser(currentUser: Models.User | undefined) {
		this.currentUser = currentUser;
	}

	@action public saveUser(user: Models.User): Models.User {
		const { users } = this;

		let savedUser = users.get(user.id);

		if (savedUser === user) return user;

		if (savedUser === undefined) {
			users.set(user.id, user);
			savedUser = users.get(user.id)!;
		}

		// Reconciliation
		for (const key of Object.keys(user)) {
			const value = user[key];

			if (value !== undefined) savedUser![key] = value;
		}

		return savedUser;
	}
}
